@@ -254,18 +254,23 @@ eta_cumu <- function(data, fcumu, cut, ...) {
   f_xyz  <- fcumu$f_xyz
   ll_fun <- fcumu$ll_fun
   eta_name <- paste0("eta_", vars[2])
-  combine_df(
+  comb_df <- combine_df(
     data.frame(t = cut),
-    select(data, one_of("id", vars))) %>%
-  unnest() %>%
-  group_by(.data$id, .data$t) %>%
-  mutate(
-    LL = ll_fun(t, !!sym(vars[1])) * 1,
-    delta = c(mean(abs(diff(!!sym(vars[1])))), abs(diff(!!sym(vars[1]))))) %>%
-  ungroup() %>%
-  filter(.data$LL != 0) %>%
-  group_by(.data$id, .data$t) %>%
-  summarize(!!eta_name :=
-    sum(.data$delta * f_xyz(.data$t, .data[[vars[1]]], .data[[vars[2]]])))
+    select(data, one_of("id", vars)))
+  if(tidyr_new_interface()) {
+    comb_df <- comb_df %>% unnest(cols = -one_of("id"))
+  } else {
+    comb_df <- comb_df %>% unnest()
+  }
+  comb_df %>%
+    group_by(.data$id, .data$t) %>%
+    mutate(
+      LL = ll_fun(t, !!sym(vars[1])) * 1,
+      delta = c(mean(abs(diff(!!sym(vars[1])))), abs(diff(!!sym(vars[1]))))) %>%
+    ungroup() %>%
+    filter(.data$LL != 0) %>%
+    group_by(.data$id, .data$t) %>%
+    summarize(!!eta_name :=
+      sum(.data$delta * f_xyz(.data$t, .data[[vars[1]]], .data[[vars[2]]])))
 
 }
