@@ -5,6 +5,8 @@ ped <- veteran %>% as_ped(Surv(time, status)~ trt + age,
   cut = c(0, 50, 100, 200, 300, 400), id = "id")
 pam <- mgcv::gam(ped_status ~ s(tend, k = 5) + trt, data = ped,
   family = poisson(), offset = offset)
+pam2 <- mgcv::gam(ped_status ~ s(tend, k = 5) + trt + s(age), data = ped,
+  family = poisson(), offset = offset)
 bam <- mgcv::bam(ped_status ~ s(tend, k = 5) + trt, data = ped,
   family = poisson(), offset = offset, method = "fREML", discrete = TRUE)
 pem <- glm(ped_status ~ 0 + interval + trt, data = ped,
@@ -43,6 +45,14 @@ test_that("hazard functions work for PAM", {
   ## simulation based ci (0.95)
   haz4 <- add_hazard(ped_info(ped), pam, ci_type = "sim")
 
+  ## hazard with reference (i.e. hazard ratio)
+  hr <- add_hazard(ped_info(ped), pam2, reference = list(age = c(30)))
+  # hazard ratio is constant as age effect not time-varying
+  expect_equal(round(hr$hazard, 3), rep(0.524, 5))
+  # hr = 1 if reference = data
+  hr2 <- ped_info(ped) %>%  add_hazard(pam2, reference = list(age = mean(.$age)))
+  expect_equal(hr2$hazard, rep(1, 5))
+
 })
 
 test_that("hazard functions work for PEM", {
@@ -115,23 +125,36 @@ test_that("cumulative hazard functions work for PEM", {
 })
 
 test_that("adding terms works for PAM", {
-  expect_data_frame(term <- add_term(ped_info(ped), bam, term = "trt"),
-    nrows = 5L, ncols = 10L)
-  expect_data_frame(term <- add_term(ped_info(ped), pam, term = "trt"),
-    nrows = 5L, ncols = 10L)
-  expect_data_frame(add_term(ped_info(ped), pam, term = "trt", relative = TRUE),
-    nrows = 5L, ncols = 10L)
-  pam$model <- NULL
-  expect_error(add_term(ped_info(ped), pam, term = "trt", relative = TRUE))
+
+  # standard
+  ndf2  <- make_newdata(ped, age = seq_range(age, 3))
+  pred2 <- ndf2 %>% add_term(pam2, term = "age")
+  expect_equal(round(pred2$fit, 3), c(0.328, -.174, .775))
+  expect_data_frame(pred2, nrows = 3L, ncols = 12L)
+  # with custom reference
+  pred2 <- ndf2 %>%
+    add_term(pam2, term = "age", reference = list(age = mean(.$age)))
+  expect_equal(round(pred2$fit, 3), c(.501, 0, .948))
+  expect_data_frame(pred2, nrows = 3L, ncols = 12L)
+  expect_equal(pred2$fit[2], 0)
+  # with overall function application
+  pred3 <- ndf2 %>% add_term(pam2, term = "age", reference = identity(.))
+  expect_equal(pred3$fit, rep(0, 3))
+  expect_data_frame(pred3, nrows = 3L, ncols = 12L)
+  expect_equal(pred3$fit, rep(0, 3))
+  # with separately created data frame
+  df_mean <- sample_info(ndf2)
+  pred4 <- ndf2 %>% add_term(pam2, term = "age", reference = df_mean)
+  expect_equal(pred4$fit, pred2$fit)
+
 })
 
 test_that("adding terms works for PEM", {
 expect_data_frame(term <- add_term(ped_info(ped), pem, term = "trt"),
     nrows = 5L, ncols = 10L)
-  expect_data_frame(add_term(ped_info(ped), pem, term = "trt", relative = TRUE),
-    nrows = 5L, ncols = 10L)
-  pem$model <- NULL
-  expect_error(add_term(ped_info(ped), pem, term = "trt", relative = TRUE))
+  expect_data_frame(ped_info(ped) %>%
+      add_term(pem, term = "trt", reference = list(trt = mean(.$trt))),
+      nrows = 5L, ncols = 10L)
 })
 
 test_that("warns about / aborts for unknown intervals", {
@@ -226,7 +249,7 @@ test_that("hazards and CI positive for type response", {
   pam <- mgcv::gam(ped_status ~ s(tend, k = 5) + trt,
     data = ped, family = poisson(), offset = offset)
   haz_test <- add_hazard(ped_info(ped), pam) %>%
-    summarize_at(c("hazard", "ci_lower", "ci_upper"), funs(any(. < 0)))
+    summarize_at(c("hazard", "ci_lower", "ci_upper"), list(~any(. < 0)))
   expect_equal(any(unlist(haz_test)), FALSE)
 
 })
